// DicksBackupView.cpp : implementation of the CDicksBackupView class
//

#include "stdafx.h"
#include "MyIncludes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupView

IMPLEMENT_DYNCREATE(CDicksBackupView, CListView)

BEGIN_MESSAGE_MAP(CDicksBackupView, CListView)
	//{{AFX_MSG_MAP(CDicksBackupView)
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_COMMAND(ID_EDIT_REMOVE, OnEditRemove)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REMOVE, OnUpdateEditRemove)
	ON_COMMAND(ID_VIEW_RESET, OnViewReset)
	ON_COMMAND(ID_EDIT_SELECT_NONE, OnEditSelectNone)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SELECT_NONE, OnUpdateEditSelectNone)
	ON_COMMAND(ID_EDIT_NOTE, OnEditNote)
	ON_UPDATE_COMMAND_UI(ID_EDIT_NOTE, OnUpdateEditNote)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_VIEW_GRAY_BAR, OnViewGrayBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GRAY_BAR, OnUpdateViewGrayBar)
	ON_COMMAND(ID_EDIT_AUTO_BACKUP, OnViewAutoBackup)
	ON_UPDATE_COMMAND_UI(ID_EDIT_AUTO_BACKUP, OnUpdateViewAutoBackup)
	ON_WM_TIMER()
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_VIEW_SHOW_ICONS, OnViewShowIcons)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHOW_ICONS, OnUpdateViewShowIcons)
	ON_COMMAND(ID_EDIT_CHANGE_DRIVE, OnChangeDrive)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CHANGE_DRIVE, OnUpdateChangeDrive)
	ON_COMMAND(ID_EDIT_SEL_ALL, OnEditSelAll)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SEL_ALL, OnUpdateEditSelAll)
	ON_COMMAND(ID_EDIT_EDIT_PATH, OnEditMoveArchivePath)
	ON_UPDATE_COMMAND_UI(ID_EDIT_EDIT_PATH, OnUpdateEditMoveArchivePath)
	ON_COMMAND(ID_EDIT_DUPLICATE, OnEditDuplicate)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DUPLICATE, OnUpdateEditDuplicate)
	ON_COMMAND(ID_EDIT_RESTORE, OnEditRestore)
	ON_UPDATE_COMMAND_UI(ID_EDIT_RESTORE, OnUpdateEditRestore)
	ON_COMMAND(ID_HELP_DOC, OnHelpDoc)
	ON_COMMAND(ID_EDIT_OPEN_SOURCE, OnEditOpenSource)
	ON_UPDATE_COMMAND_UI(ID_EDIT_OPEN_SOURCE, OnUpdateEditOpenSource)
	//}}AFX_MSG_MAP
	ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, OnCustomDraw)
	ON_COMMAND_RANGE(ID_VIEW_NAME, ID_VIEW_TYPE_NAME, OnShowCol)
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_NAME, ID_VIEW_TYPE_NAME, OnUpdateShowCol)
	ON_COMMAND_RANGE(ID_VIEW_LITTLE_ENDIAN, ID_VIEW_AGE, OnChangeDate)
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_LITTLE_ENDIAN, ID_VIEW_AGE, OnUpdateChangeDate)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_NUM_FILES, OnUpdateNumFiles)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_NUM_BYTES, OnUpdateNumBytes)
	ON_COMMAND_RANGE(ID_VIEW_SMALL_FONT, ID_VIEW_LARGE_FONT, OnChangeFont)
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_SMALL_FONT, ID_VIEW_LARGE_FONT, OnUpdateChangeFont)
	ON_COMMAND_RANGE(ID_EDIT_SOURCE_EXPLORER, ID_EDIT_MANAGER_EXPLORER, OnEditExplorer)
	ON_UPDATE_COMMAND_UI_RANGE(ID_EDIT_SOURCE_EXPLORER, ID_EDIT_MANAGER_EXPLORER, OnUpdateEditExplorer)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_NUM_OBSELETE, OnUpdateNumObselete)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_NUM_PROBLEMS, OnUpdateNumProblems)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupView construction/destruction

CDicksBackupView::CDicksBackupView()
{
	m_pFont = NULL;
	m_pImageList = NULL;
}

CDicksBackupView::~CDicksBackupView()
{
	int nCols = m_hdrExtraDataPtrArray.GetSize();

	// release memory for header lParams
	while(--nCols >= 0)
	{
		delete m_hdrExtraDataPtrArray[nCols];
	}

	if(m_pFont) delete m_pFont;
}

BOOL CDicksBackupView::PreCreateWindow(CREATESTRUCT& cs)
{
	if(!CListView::PreCreateWindow(cs))
	{
		return FALSE;
	}

	cs.style |= (LVS_REPORT | LVS_SHOWSELALWAYS);

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CDicksBackupView drawing
void CDicksBackupView::SetFont(int fontPtSize)
{
	static char procName[] = _T("Change Font");

	CFont * pFont = new CFont;
	CDC * pDc = m_pListCtrl->GetDC();
	ASSERT(pDc);

	if(pFont && pFont->CreatePointFont(10 * fontPtSize, DFLT_FONT_FACE_NAME, pDc))
	{
		if(m_pFont != NULL)
		{
			delete m_pFont;
		}

		m_pListCtrl->SetFont(pFont, TRUE);
		m_pFont = pFont;

		// scale the collumn width to the font size
		for(UINT i = 0; i < NCOLS; i++)
		{
			m_pListCtrl->SetColumnWidth(i, m_pListCtrl->GetColumnWidth(i) * fontPtSize / m_fontPtSize);
		}

		m_fontPtSize = fontPtSize;
	}
	else
	{
		MessageBox(_T("Problem creating a new font"), procName, MB_ICONERROR);
	}

	m_pListCtrl->ReleaseDC(pDc);
}

void CDicksBackupView::OnChangeFont(UINT nID)
{
	UINT nIndex = nID - ID_VIEW_SMALL_FONT;
	// change toolbar height
	((CMainFrame *)AfxGetMainWnd())->SetToolBarSize(nIndex);

	SetFont(nPointSize[nIndex]);

	RefreshEntireList();

	Invalidate();

}

void CDicksBackupView::OnUpdateChangeFont(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_fontPtSize == nPointSize[pCmdUI->m_nID - ID_VIEW_SMALL_FONT]);
}

void CDicksBackupView::OnInitialUpdate()
{
	CListView::OnInitialUpdate();

	m_pDoc = GetDocument();
	ASSERT(m_pDoc->IsKindOf(RUNTIME_CLASS(CDicksBackupDoc)));
	m_pListCtrl = &GetListCtrl();
	ASSERT_VALID(m_pListCtrl);

	// first time at program start
	if(GetColumns() == 0)
	{
		m_pListCtrl->SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES | 
									  LVS_EX_HEADERDRAGDROP /* | LVS_EX_SUBITEMIMAGES */);

		CDicksBackupApp * pApp = (CDicksBackupApp *)AfxGetApp();

		// start out by setting previous font == next font
		m_fontPtSize = pApp->GetProfileInt(settings, nFontPtSize, DFLT_FONT_PT_SIZE);
		SetFont(m_fontPtSize);

		m_bGrayBar = pApp->GetProfileInt(settings, bGrayBar, FALSE);
		m_nDateFmt = pApp->GetProfileInt(settings, nDateFmt, MIDDLE_ENDIAN);
		m_autoBackupTimeHours = pApp->GetProfileInt(settings, nAutoBackupTimeHr, DFLT_AUTO_BACKUP_TIMER_HOURS);
		m_bShowIcons = pApp->GetProfileInt(settings, bShowIcons, TRUE);
		
		CString aStr;
		int pOrderIndex[NCOLS];

		for(UINT i = 0; i < NCOLS; i++)
		{
			// get column widths forn registry
			aStr.Format(_T("nColWidth%d"), i);
			UINT width = pApp->GetProfileInt(settings, aStr, m_pListCtrl->GetStringWidth(columnName[i]) * 2);
			
			// column with in low word and order index in high word
			pOrderIndex[i] = width >> 16;
			width &= 0XFFFF;

			m_pListCtrl->InsertColumn(i, columnName[i], LVCFMT_LEFT, width); 
			// get show hide columns from registry
			CHdrExtraData * pHdrLparam = new CHdrExtraData(i);
			m_hdrExtraDataPtrArray.Add(pHdrLparam);
			pHdrLparam->m_bShowColumn = width > 0 ? TRUE : FALSE;
		}

		m_pListCtrl->GetHeaderCtrl()->SetOrderArray(NCOLS, pOrderIndex);
	}

	m_pListCtrl->SetBkColor(m_pDoc->GetPathName().IsEmpty() ? ::GetSysColor(NO_DOC_BKGND_COLOR) : NOT_BACKED_UP_BKGND_LIST_COLOR);

	RefreshEntireList();

	// start with refresh timer off
	KillTimer(AUTO_BACKUP_TIMER_ID);
	m_bAutoBackup = FALSE;

	UINT nCols = m_hdrExtraDataPtrArray.GetSize();
	// set initial sort ASCENDING;
	for(UINT i = 0; i < nCols; i++)
	{
		m_hdrExtraDataPtrArray[i]->m_nSortOrder = ASCENDING;
	}
}

// helper function
void CDicksBackupView::CreateImageList()
{
	if(m_pImageList != NULL)
	{
		delete m_pImageList;
	}

	m_pImageList = new CImageList;

	if(m_pImageList)
	{
		int imageSize = 1; // used when we don't want to show the icons in the NAME column

		if(m_bShowIcons)
		{
			// select image X = Y-size according to font height
			CDC * pDc =	m_pListCtrl->GetDC();
			CFont * pFont = m_pListCtrl->GetFont();
			CFont * pOldFont = pDc->SelectObject(pFont);
			TEXTMETRIC tm;
			pDc->GetTextMetrics(&tm);
			pDc->SelectObject(pOldFont);
			imageSize = tm.tmHeight;
			m_pListCtrl->ReleaseDC(pDc);
		}

		m_pImageList->Create(imageSize, imageSize, ILC_COLOR32 /* | ILC_MASK */, 20, 10);
	}
}

void CDicksBackupView::RefreshEntireList()
{
	m_pListCtrl->DeleteAllItems();

	UINT nItems = m_pDoc->m_itemDataPtrArray.GetSize();

	if(nItems > 0)
	{
		m_pListCtrl->SetItemCount(nItems);

		CreateImageList();

		m_pListCtrl->SetImageList(m_pImageList, LVSIL_SMALL);

		for(UINT i = 0; i < nItems; i++)
		{
			AppendItemToList(m_pDoc->m_itemDataPtrArray[i]);
		}
	}
}

void CDicksBackupView::AppendItemToList(CItemData * pItemData)
{
	ASSERT(pItemData);
	if(pItemData)
	{
		int iItem = m_pListCtrl->GetItemCount();
			
		int nIndex = m_pImageList ? m_pImageList->Add(pItemData->m_hIcon): 0;
		m_pListCtrl->InsertItem(iItem, _T(""), nIndex);

		m_pListCtrl->SetItemData(iItem, (DWORD)pItemData);
		UpdateRow(iItem);
	}
}

void CDicksBackupView::UpdateRow(UINT item)
{
	CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(item);

	m_pListCtrl->SetItemText(item, FILE_NAME, CString(_T(" " ) + pItemData->GetName()));
	m_pListCtrl->SetItemText(item, FILE_TYPE, pItemData->GetType());
	m_pListCtrl->SetItemText(item, FILE_SIZE, pItemData->FormatSize());
	m_pListCtrl->SetItemText(item, LAST_MODIFIED, pItemData->FormatDate(m_nDateFmt));
	m_pListCtrl->SetItemText(item, SOURCE_PATH, pItemData->m_sourcePathStr);
	m_pListCtrl->SetItemText(item, ARCHIVE_PATH, pItemData->m_archivePathStr);

	CString aStr(_T(""));

	if(pItemData->m_nlastError > 0) // format using sustem errno message
	{
		LPVOID lpMsgBuf;
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
					  FORMAT_MESSAGE_FROM_SYSTEM | 
					  FORMAT_MESSAGE_IGNORE_INSERTS,
					  NULL,
					  pItemData->m_nlastError,
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  (LPTSTR) &lpMsgBuf,
					  0,
					  NULL);

		aStr.Format(_T("%s %s"), sourceArchiveStr[pItemData->m_nExceptionIndicator], (LPCTSTR)lpMsgBuf);
		LocalFree(lpMsgBuf);
		aStr.TrimRight(); // remove possible CR-LF at end
		m_pListCtrl->SetItemText(item, RESULTS, aStr);
	}

	m_pListCtrl->SetItemText(item, RESULTS, aStr);
	m_pListCtrl->SetItemText(item, OBSOLETE, pItemData->GetTimeOutOfDate());
	m_pListCtrl->SetItemText(item, NOTES, pItemData->m_notesStr);
	m_pListCtrl->SetItemText(item, FOLDER, pItemData->GetFolder());
	m_pListCtrl->SetItemText(item, DRIVE, pItemData->GetDrive());
	m_pListCtrl->SetItemText(item, TYPE_NAME, pItemData->m_typeName);
	m_pListCtrl->Update(item);
}

void CDicksBackupView::OnDestroy() 
{
	CDicksBackupApp * pApp = (CDicksBackupApp *)AfxGetApp();
	// save to registry
	pApp->WriteProfileInt(settings, nFontPtSize,  m_fontPtSize);
	pApp->WriteProfileInt(settings, bGrayBar,  m_bGrayBar);
	pApp->WriteProfileInt(settings, nDateFmt, m_nDateFmt);
	pApp->WriteProfileInt(settings, nAutoBackupTimeHr, m_autoBackupTimeHours);
	pApp->WriteProfileInt(settings, bShowIcons, m_bShowIcons);
		 
	int pOrderIndex[NCOLS] = {0};
	m_pListCtrl->GetHeaderCtrl()->GetOrderArray(pOrderIndex, NCOLS);
	CString aStr;

	// save to registry column width in low word and order index in high word to registry
	for(UINT i = 0; i < NCOLS; i++)
	{
		aStr.Format(_T("nColWidth%d"), i);
		pApp->WriteProfileInt(settings, aStr, m_pListCtrl->GetColumnWidth(i) | (pOrderIndex[i] << 16));
	}

	// kill refresh timer
	if(m_bAutoBackup)
	{
		KillTimer(AUTO_BACKUP_TIMER_ID);
	}

	if(m_pImageList != NULL)
	{
		delete m_pImageList;
	}

	CListView::OnDestroy();
}
	
/////////////////////////////////////////////////////////////////////////////
// CDicksBackupView diagnostics

#ifdef _DEBUG
void CDicksBackupView::AssertValid() const
{
	CListView::AssertValid();
}

void CDicksBackupView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CDicksBackupDoc* CDicksBackupView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDicksBackupDoc)));
	return (CDicksBackupDoc*)m_pDocument;
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CDicksBackupView message handlers
void CDicksBackupView::OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LPNMLVCUSTOMDRAW lplvcd = (LPNMLVCUSTOMDRAW)pNMHDR;
	if(lplvcd->nmcd.dwDrawStage == CDDS_PREPAINT)
	{
	    *pResult = CDRF_NOTIFYITEMDRAW;
	}
    else if (lplvcd->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
	{
		int item = (int)lplvcd->nmcd.dwItemSpec;
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(item); 

		lplvcd->clrText = pItemData->m_lineFgColor;

		if(m_bGrayBar)
		{
			lplvcd->clrTextBk = (item & 1) ? LT_GRAY : WHITE;
		}
		else
		{
			lplvcd->clrTextBk = pItemData->m_lineBkColor;
		}

		*pResult = CDRF_DODEFAULT;			// color whole line
//	    *pResult = CDRF_NOTIFYSUBITEMDRAW;	// color subitem
	}
    else if (lplvcd->nmcd.dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
    {
/*
        // test data for coloring individual list cells
		int pos = (int)lplvcd->nmcd.dwItemSpec;
		int sub = (int)lplvcd->iSubItem;

		TRACE2("Pos %d Sub %x\n", pos, sub);

		if(pos == 3 && sub == 2)
		{
			lplvcd->clrText = PROBLEM_LIST_COLOR;
			lplvcd->clrTextBk = NORMAL_LIST_BKGND_COLOR;
		}
		else
		{
			lplvcd->clrText = NOT_BACKED_UP_LIST_COLOR;
			lplvcd->clrTextBk = NORMAL_LIST_BKGND_COLOR;
		}

		*pResult = CDRF_DODEFAULT;
*/
	}

}

// sorting on columns call back proc
static int CALLBACK MyCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParm3)
{
	CHdrExtraData * pHrdExtraData = (CHdrExtraData *)lParm3;

	if(pHrdExtraData->m_nSortOrder == ASCENDING)
	{
		Swap(lParam1, lParam2);
	}

	CItemData * pData1 = (CItemData *)lParam1;
	CItemData * pData2 = (CItemData *)lParam2;

	switch (pHrdExtraData->m_index)
	{
		case FILE_NAME:
			return stricmp(pData1->GetName(), pData2->GetName());
		case FILE_TYPE:
			return stricmp(pData1->GetType(), pData2->GetType());
		case FILE_SIZE:
			return pData1->m_nSourceSize - pData2->m_nSourceSize;
		case LAST_MODIFIED:
			return pData2->m_modifiedTime - pData1->m_modifiedTime;
		case SOURCE_PATH:
			return stricmp(pData1->m_sourcePathStr, pData2->m_sourcePathStr);
		case ARCHIVE_PATH:
			return stricmp(pData1->m_archivePathStr, pData2->m_archivePathStr);
		case RESULTS:
			return pData1->m_nlastError - pData2->m_nlastError;
		case OBSOLETE:
			return pData2->m_timeOutOfDate - pData1->m_timeOutOfDate;
		case NOTES:
			return stricmp(pData2->m_notesStr, pData1->m_notesStr);
		case FOLDER:
			return stricmp(pData1->GetFolder(), pData2->GetFolder());
		case DRIVE:
			return stricmp(pData1->GetDrive(), pData2->GetDrive());
		case TYPE_NAME:
			return stricmp(pData1->m_typeName, pData2->m_typeName);
	}

	ASSERT(FALSE);
	return 0;
}

// alternate clicks sorts column ascending then decending
void CDicksBackupView::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if(GetRows() >0)
	{
		NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
		
		DWORD nCol = pNMListView->iSubItem;

		// toggle column's sort direction
		m_hdrExtraDataPtrArray[nCol]->m_nSortOrder = 
			(m_hdrExtraDataPtrArray[nCol]->m_nSortOrder == ASCENDING) ? 
				DECENDING : ASCENDING;

		// pass extra header data to compare proc as LPARAM
			m_pListCtrl->SortItems(MyCompareProc, (LPARAM)m_hdrExtraDataPtrArray[nCol]);

		GetDocument()->SetModifiedFlag(TRUE);
	}

	*pResult = 0;
}

UINT CDicksBackupView::GetRows()
{
	return m_pListCtrl->GetItemCount();
}

UINT CDicksBackupView::GetColumns()
{
	return m_pListCtrl->GetHeaderCtrl()->GetItemCount();
}

void CDicksBackupView::OnShowCol(UINT nID)
{
	UINT nCol = nID - ID_VIEW_NAME;
	m_hdrExtraDataPtrArray[nCol]->m_bShowColumn = !(m_hdrExtraDataPtrArray[nCol]->m_bShowColumn);

	// set col width to max of header or column text width. Add 20 pixels for good measure
	if(m_hdrExtraDataPtrArray[nCol]->m_bShowColumn)
	{
		int width = m_pListCtrl->GetStringWidth(columnName[nCol]);

		UINT nRows = GetRows();

		for(UINT i = 0; i < nRows; i++)
		{		
			width = max(width, m_pListCtrl->GetStringWidth(m_pListCtrl->GetItemText(i, nCol)));
		}

		m_pListCtrl->SetColumnWidth(nCol, width + 20); 
	}
	else
	{
		m_pListCtrl->SetColumnWidth(nCol, 0);	
	}
}

void CDicksBackupView::OnUpdateShowCol(CCmdUI* pCmdUI)
{
	UINT nCol = pCmdUI->m_nID - ID_VIEW_NAME;
	pCmdUI->SetCheck(m_hdrExtraDataPtrArray[nCol]->m_bShowColumn);
}

void CDicksBackupView::OnViewReset()  
{
	int nCol = GetColumns();
	int pOrderIndex[NCOLS];
	ASSERT(nCol == NCOLS);

	while(--nCol >= 0)
	{
		pOrderIndex[nCol] = nCol;
		m_hdrExtraDataPtrArray[nCol]->m_bShowColumn = TRUE;
		m_pListCtrl->SetColumnWidth(nCol, m_pListCtrl->GetStringWidth(columnName[nCol]) * 2);
	}

	m_pListCtrl->GetHeaderCtrl()->SetOrderArray(NCOLS, pOrderIndex);
	Invalidate();
}

void CDicksBackupView::OnChangeDate(UINT nID)
{
	m_nDateFmt = nID - ID_VIEW_LITTLE_ENDIAN;

	int nItems = GetRows();

	while(--nItems >= 0)
	{
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItems);
		m_pListCtrl->SetItemText(nItems, LAST_MODIFIED, pItemData->FormatDate(m_nDateFmt));
	}
}

void CDicksBackupView::OnUpdateChangeDate(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(pCmdUI->m_nID - ID_VIEW_LITTLE_ENDIAN == m_nDateFmt);
}

void CDicksBackupView::OnUpdateNumFiles(CCmdUI* pCmdUI)
{
	CString aStr;
	aStr.Format("\t%d Files", GetRows());
	pCmdUI->SetText(aStr);
}
 
void CDicksBackupView::OnUpdateNumObselete(CCmdUI* pCmdUI)
{
	int nItems = GetRows(); // must be a int not UINT
	int nObsolete = 0;

	while(--nItems >= 0)
	{
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItems);

		if(pItemData->m_lineFgColor == NOT_BACKED_UP_LIST_COLOR)
		{
			nObsolete++;
		}
	}

	CString aStr(_T("\tAll Backed Up"));

	if(nObsolete > 0)
	{
		aStr.Format("\t%d Obselete", nObsolete);
	}

	pCmdUI->SetText(aStr);
}

void CDicksBackupView::OnUpdateNumProblems(CCmdUI* pCmdUI)
{
	int nItems = GetRows(); // must be a int not UINT
	int nProblems = 0;

	while(--nItems >= 0)
	{
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItems);

		if(pItemData->m_lineFgColor == PROBLEM_LIST_COLOR)
		{
			nProblems++;
		}
	}

	CString aStr(_T("\tNo Problems"));

	if(nProblems > 0)
	{
		aStr.Format("\t%d Problems", nProblems);
	}

	pCmdUI->SetText(aStr);
}

void CDicksBackupView::OnUpdateNumBytes(CCmdUI* pCmdUI)
{
	int nItems = GetRows(); // must be a int not UINT
	double nBytes = 0.0;
	
	while(--nItems >= 0)
	{
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItems);
		nBytes += pItemData->m_nSourceSize;
	}

	UINT index = 0;

	if(nBytes >= 1e3) // Killo
	{
		index++;
		nBytes /= 1e3;
	}

	if(nBytes >= 1e3) // Meg
	{
		index++;
		nBytes /= 1e3;
	}

	if(nBytes >= 1e3) // Gig
	{
		index++;
		nBytes /= 1e3;
	}

	CString aStr;
	aStr.Format("\t%.0f %s", nBytes, byteStr[index]);
	pCmdUI->SetText(aStr);
}

void CDicksBackupView::OnEditSelectNone()
{
	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();

	while (pos)
	{
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);
		m_pListCtrl->SetItemState(nItem, 0, LVIS_SELECTED);
	}
}

void CDicksBackupView::OnUpdateEditSelectNone(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() > 0);
}

void CDicksBackupView::OnEditNote() 
{
	static const char procStr[] = _T("Edit Selected Notes");

	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();

	if(pos)
	{
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);

		CEnterStrDlg aDlg;
		aDlg.m_captionStr = procStr;
		aDlg.m_editStr = m_pListCtrl->GetItemText(nItem, NOTES); // notes column for first selected row

		if(IDOK == aDlg.DoModal())
		{
			m_pListCtrl->SetItemText(nItem, NOTES, aDlg.m_editStr);
			CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItem);
			pItemData->m_notesStr = aDlg.m_editStr;

			while(pos)
			{
				nItem = m_pListCtrl->GetNextSelectedItem(pos);
				m_pListCtrl->SetItemText(nItem, NOTES, aDlg.m_editStr);
				pItemData = (CItemData *)m_pListCtrl->GetItemData(nItem);
				pItemData->m_notesStr = aDlg.m_editStr;
			}

			m_pDoc->SetModifiedFlag(TRUE);
		}
	}	
}

void CDicksBackupView::OnUpdateEditNote(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() > 0);
}

void CDicksBackupView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CMenu aMenu;
	aMenu.LoadMenu(IDR_EDIT_POPUP);
	CMenu * pPopup = aMenu.GetSubMenu(0);
	ASSERT_VALID(pPopup);
	CRect clientRect;
	GetClientRect(&clientRect);
	ClientToScreen(&clientRect);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, point.x, point.y, AfxGetMainWnd(), NULL);
}

void CDicksBackupView::OnViewGrayBar() 
{
	m_bGrayBar = !m_bGrayBar;
	Invalidate();
}

void CDicksBackupView::OnUpdateViewGrayBar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bGrayBar);	
}

void CDicksBackupView::OnEditExplorer(UINT nID) 
{
	static char procName[] = _T("Open in Explorer");

	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();
	CString pathStr;
	// get path
	switch (nID)
	{
		case ID_EDIT_SOURCE_EXPLORER:
			if(pos) pathStr = m_pListCtrl->GetItemText(m_pListCtrl->GetNextSelectedItem(pos), SOURCE_PATH);
			break;
		case ID_EDIT_ARCHIVE_EXPLORER:
			if(pos) pathStr = m_pListCtrl->GetItemText(m_pListCtrl->GetNextSelectedItem(pos), ARCHIVE_PATH);
			break;
		case ID_EDIT_MANAGER_EXPLORER:
			pathStr = m_pDoc->GetPathName();
			break;
	}

	if(!pathStr.IsEmpty())
	{
		// hilight directory and file in Windows Explorer
		CString runStr(_T("explorer.exe /e,/select,"));
		runStr += _T('\"') + pathStr + _T('\"');

		UINT retVal = 0;

		if((retVal = ::WinExec(runStr, SW_SHOW) <= 31))
		{
			CString aStr(_T("Unknown Error while starting Explorer"));

			if(retVal == ERROR_BAD_FORMAT)
			{
				aStr = _T("Explorer program is invalid");
			}
			else if(retVal = ERROR_FILE_NOT_FOUND)
			{
				aStr = _T("Explorer program was not found.");
			}
			else if(retVal = ERROR_PATH_NOT_FOUND)
			{
				aStr = _T("Explorer program path was not found.");
			}

			MessageBox(aStr, procName, MB_ICONERROR);
		}
	}
}

void CDicksBackupView::OnUpdateEditExplorer(CCmdUI* pCmdUI) 
{
	BOOL bEnable = FALSE;

	if(m_pListCtrl->GetSelectedCount() == 1) // only one at a time shown in Explorer
	{
		POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();
		// first selected 
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItem);

		if(pCmdUI->m_nID == ID_EDIT_SOURCE_EXPLORER)
		{
			bEnable = pItemData->m_nExceptionIndicator == SOURCE ? FALSE : TRUE;
		}
		else // ID_EDIT_ARCHIVE_EXPLORER
		{
			bEnable = pItemData->m_nExceptionIndicator == ARCHIVE ? FALSE : TRUE;
		}
	}  

	pCmdUI->Enable(bEnable || 
		          (pCmdUI->m_nID == ID_EDIT_MANAGER_EXPLORER && !m_pDoc->GetPathName().IsEmpty()));	
}

// toggle auto refresh timer
void CDicksBackupView::OnViewAutoBackup() 
{  
 	m_bAutoBackup = !m_bAutoBackup;

	if(m_bAutoBackup)
	{
		CEnterStrDlg aDlg;

 		aDlg.m_captionStr = _T("Auto Backup Time Hours");
		aDlg.m_editStr.Format(_T("%u"), m_autoBackupTimeHours);
		aDlg.m_addStyles = ES_NUMBER;

		if(IDOK == aDlg.DoModal())
		{
			m_autoBackupTimeHours = max(:: atoi(aDlg.m_editStr), 1); // zero minutes not acceptable

			SetTimer(AUTO_BACKUP_TIMER_ID, 1000 * m_autoBackupTimeHours * 60, NULL); 
		}
		else
		{
			m_bAutoBackup = FALSE;
		}
	}
	else
	{
		KillTimer(AUTO_BACKUP_TIMER_ID);
	}
}

void CDicksBackupView::OnUpdateViewAutoBackup(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bAutoBackup);
	pCmdUI->Enable(GetRows() > 0);

	CString aStr(_T("Auto Backup (off)"));
	if(m_bAutoBackup)
	{
		aStr.Format(_T("Auto Backup (%d hour)"), m_autoBackupTimeHours);
	}
	pCmdUI->SetText(aStr);
}

void CDicksBackupView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent == AUTO_BACKUP_TIMER_ID)
	{
		// let document class do backup all
		AfxGetMainWnd()->FlashWindow(TRUE);
		GetDocument()->m_bSilent = TRUE;
		AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_EDIT_DO_BACKUP);
		GetDocument()->m_bSilent = FALSE;
		AfxGetMainWnd()->FlashWindow(TRUE);
	}
	else
	{
		CListView::OnTimer(nIDEvent);
	}
}

void CDicksBackupView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	SendMessage(WM_COMMAND, ID_EDIT_SOURCE_EXPLORER);
	CListView::OnLButtonDblClk(nFlags, point);
}


void CDicksBackupView::OnViewShowIcons() 
{
	m_bShowIcons = !m_bShowIcons;
	RefreshEntireList();	
}

void CDicksBackupView::OnUpdateViewShowIcons(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bShowIcons);	
}

void CDicksBackupView::OnEditSelAll() 
{
	int nItems = m_pListCtrl->GetItemCount();

	for(int i = 0; i < nItems; i++)
	{
		m_pListCtrl->SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
	}
}

void CDicksBackupView::OnUpdateEditSelAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetItemCount());	
}

void CDicksBackupView::OnChangeDrive() 
{
	static char const procStr[] = _T("Change Archive Drive Letter");
	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();

	if(pos)
	{
		CEnterStrDlg aDlg;
		aDlg.m_captionStr.Format(_T("%s (%d Files Selected)"), procStr, m_pListCtrl->GetSelectedCount());
		// get drive letter of first selected as a example for dialog
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);
		pos = m_pListCtrl->GetFirstSelectedItemPosition(); // reset selection position back to first
		_splitpath(m_pListCtrl->GetItemText(nItem, ARCHIVE_PATH), aDlg.m_editStr.GetBuffer(_MAX_DRIVE), NULL, NULL, NULL);
		aDlg.m_editStr.ReleaseBuffer();

		while(IDOK == aDlg.DoModal())
		{
			aDlg.m_editStr.TrimRight();
			aDlg.m_editStr.TrimLeft();
			aDlg.m_editStr.MakeUpper();

			if(aDlg.m_editStr.GetLength() > 0)
			{
				if(aDlg.m_editStr[aDlg.m_editStr.GetLength() - 1] != _T(':'))
				{
					aDlg.m_editStr += _T(':');
				}
			
				if(aDlg.m_editStr.GetLength() == 2 && ::isalpha(aDlg.m_editStr[0]))
				{
					while(pos)
					{
						int nItem = m_pListCtrl->GetNextSelectedItem(pos);
						CItemData * pItemDate = (CItemData *)m_pListCtrl->GetItemData(nItem);
						pItemDate->ChangeDrive(aDlg.m_editStr);
						m_pListCtrl->SetItemState(nItem, 0, LVIS_SELECTED);
					}

					AfxGetMainWnd()->PostMessage(WM_COMMAND, ID_EDIT_REFRESH);
					m_pDoc->SetModifiedFlag(TRUE);
					break;
				}
				else
				{
					MessageBeep(0xFFFFFFFF);
					aDlg.m_editStr = _T("Enter a single letter for the drive!");
				}
			}
		}
	}
}

void CDicksBackupView::OnUpdateChangeDrive(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetFirstSelectedItemPosition() > 0);	
}

void CDicksBackupView::OnEditMoveArchivePath() 
{
	static char const procStr[] = _T("Move Archive's File");

	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();

	if(pos)
	{
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);

		if(pos == 0) // Allow only one item selected
		{
			CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItem);

			CEnterStrDlg aDlg;
			aDlg.m_captionStr = _T("Move Archive to new Folder and/or change Filename");
			aDlg.m_editStr = pItemData->m_archivePathStr;
			aDlg.m_bShowBrowseButton = TRUE;
				
			if(IDOK == aDlg.DoModal())
			{
				CString & archivePathStr = aDlg.m_editStr;
				aDlg.m_editStr.TrimRight();
				aDlg.m_editStr.TrimLeft();

				if(archivePathStr.CompareNoCase(pItemData->m_archivePathStr) != 0) // don't archive to previous archive
				{
					if(archivePathStr.CompareNoCase(pItemData->m_sourcePathStr) != 0) // don't archive to source's path
					{
 						BOOL bFailIfExist = TRUE;

						for(int i = 0; i < 2; i++) // try at most twice
						{
							// move = copy + delete
							if(::CopyFile(pItemData->m_archivePathStr, archivePathStr, bFailIfExist))
							{
								::DeleteFile(pItemData->m_archivePathStr);
								pItemData->m_archivePathStr = archivePathStr;
								UpdateRow(nItem);
								m_pDoc->SetModifiedFlag(TRUE);
								break;
							}
							else if(i == 0)
							{
								char drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
								::_splitpath(archivePathStr, drive, dir, fname, ext);

								CString aStr;
								aStr.Format(_T("%s%s\n\nalready exists in directory\n\n%s%s\n\nCopy over it?"),
											fname, ext, drive, dir);

								if(IDYES == MessageBox(aStr, procStr, MB_ICONQUESTION | MB_YESNO))
								{
									bFailIfExist = FALSE; // now write over existing
								}
								else
								{
									break; // quit
								}
							}
							else if(i == 1) // failure to overwtire
							{
								MessageBox(GetLastErrorMessage(), procStr, MB_ICONINFORMATION);
							}
						} // for(int i = 0; i < 2; i++)
					} // if(sourcePathStr.CompareNoCase(archivePathStr) != 0)
					else
					{
						MessageBox(_T("Source and new Archive paths are identical!\n\n")
								   _T("Operation will be aborted."), procStr, MB_ICONWARNING); 
					}
				} // if(archivePathStr.CompareNoCase(pItemData->m_archivePathStr) != 0)
				else
				{
					MessageBox(_T("New and previous Archive paths are identical!\n\n")
							   _T("Operation will be aborted."), procStr, MB_ICONWARNING); 
				}
			}
		} // if(pos == 0)
	} // if(pos) 
}

void CDicksBackupView::OnUpdateEditMoveArchivePath(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() == 1); 	
}

// Crerate a new backup directive with the same source but a different archive path
void CDicksBackupView::OnEditDuplicate() 
{
	static char const procStr[] = _T("Additional Backup to Different Path");

	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();

	if(pos > 0)
	{
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);

		if(pos == 0) // Allow only one item selected
		{
			CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItem);
			
			CEnterStrDlg aDlg;
			aDlg.m_captionStr = _T("Enter Additional Archive Path");
			aDlg.m_editStr = pItemData->m_archivePathStr;
			aDlg.m_bShowBrowseButton = TRUE;

			if(IDOK == aDlg.DoModal())
			{
				CString & archivePathStr = aDlg.m_editStr;
				archivePathStr.TrimRight();
				archivePathStr.TrimLeft();

				if(archivePathStr.CompareNoCase(pItemData->m_archivePathStr) != 0) // don't archive to previous archive
				{
					CString & sourcePathStr = pItemData->m_sourcePathStr;

					if(sourcePathStr.CompareNoCase(archivePathStr) != 0) // don't archive to source's path
					{
						BOOL bFailIfExist = TRUE;

						for(int i = 0; i < 2; i++) // try at most twice
						{
							if(::CopyFile(sourcePathStr, archivePathStr, bFailIfExist))
							{
								// put duplicate directive in the item data array
								CItemData * pDupItemData = new CItemData(sourcePathStr, archivePathStr, pItemData->m_notesStr);
								m_pDoc->m_itemDataPtrArray.Add(pDupItemData);
								// get icon for duplicate
								LVITEM lvItem = {LVIF_IMAGE, nItem, 0};
								m_pListCtrl->GetItem(&lvItem);
								// put duplicated directive in the list below the original
								nItem++;
								m_pListCtrl->InsertItem(nItem , _T(""), lvItem.iImage);
								m_pListCtrl->SetItemData(nItem, (DWORD)pDupItemData);

								UpdateRow(nItem);
								m_pDoc->SetModifiedFlag(TRUE);
								break;
							}
							else if(i == 0)
							{
								char drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
								::_splitpath(archivePathStr, drive, dir, fname, ext);

								CString aStr;
								aStr.Format(_T("%s%s\n\nalready exists in directory\n\n%s%s\n\nCopy over it?"),
											fname, ext, drive, dir);

								if(IDYES == MessageBox(aStr, procStr, MB_ICONQUESTION | MB_YESNO))
								{
									bFailIfExist = FALSE; // now write over existing
								}
								else
								{
									break; // quit
								}
							}
							else if(i == 1) // failure to overwtire
							{
								MessageBox(GetLastErrorMessage(), procStr, MB_ICONINFORMATION);
							}
						} // for(int i = 0; i < 2; i++)
					} // if(sourcePathStr.CompareNoCase(archivePathStr) != 0)
					else
					{
						MessageBox(_T("Source and new Archive paths are identical!\n\n")
								   _T("Operation will be aborted."), procStr, MB_ICONWARNING); 
					}
				} // if(archivePathStr.CompareNoCase(pItemData->m_archivePathStr) != 0)
				else
				{
					MessageBox(_T("New and previous Archive paths are identical!\n\n")
							   _T("Operation will be aborted."), procStr, MB_ICONWARNING); 
				}
			}// if(IDOK == aDlg.DoModal())
		} // if(pos == 0)
	} // if(pos > 0)
}

void CDicksBackupView::OnUpdateEditDuplicate(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() == 1); 	
}

void CDicksBackupView::OnEditRemove() 
{
	UINT nCount = m_pListCtrl->GetSelectedCount();
	CString procStr;
	procStr.Format(_T("Renove %d File%s from Backup"), nCount, nCount == 1 ? _T("") : _T("s"));

	CString msgStr;
	msgStr.Format(_T("Do you also want to delete the archive file%s?"), nCount == 1 ? _T("") : _T("s"));
	UINT retVal = MessageBox(msgStr, procStr, MB_ICONQUESTION | MB_YESNOCANCEL | MB_DEFBUTTON3);

	if(retVal == IDYES || retVal == IDNO)
	{
		int nItems = GetRows();
		UINT nDeleated = 0;

		// remove from bottom up so we don't change items index above deletion position
		while(--nItems >= 0)
		{
			if(LVIS_SELECTED == m_pListCtrl->GetItemState(nItems, LVIS_SELECTED))
			{
				CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItems);				

				UINT nCount = m_pDoc->m_itemDataPtrArray.GetSize();

				for(UINT i = 0; i < nCount; i++)
				{
					// match CItemData pointers to find class data for deletion
					if(pItemData == m_pDoc->m_itemDataPtrArray[i])
					{
						if(retVal == IDYES)
						{
							DeleteFile(pItemData->m_archivePathStr);
						}

						m_pListCtrl->DeleteItem(nItems);
						delete pItemData;
						m_pDoc->m_itemDataPtrArray.RemoveAt(i);

						break;
					}
				}

				nDeleated++;
			}
		}

		if(nDeleated > 0)
		{
			m_pDoc->SetModifiedFlag(TRUE);
		}
	}
}

void CDicksBackupView::OnUpdateEditRemove(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() > 0);	
}

void CDicksBackupView::OnEditRestore() 
{
	UINT nSel = m_pListCtrl->GetSelectedCount();
	CString procStr;
	procStr.Format(_T("Restore %d File%c"), nSel, nSel == 1 ? _T(' ') : _T('s'));

	CString msgStr;
	msgStr.Format(_T("Do you also want make a copy of\nthe source before restoring %s file?"), nSel == 1 ? _T("the") : _T("each"));
	UINT retVal = MessageBox(msgStr, procStr, MB_ICONQUESTION | MB_YESNOCANCEL | MB_DEFBUTTON3);

	if(retVal == IDYES || retVal == IDNO)
	{
		UINT nRestored = 0,
			 nCopyMade = 0;
		char drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];

		{ // put CProgressCtrl in its own stack frame
			PROGRESS(nSel, 1, _T("Restoring Files, Please Wait"));
			CString sourceCopyPathStr;
			BOOL bShowProblemMessage = IDOK;
			UINT nItems = GetRows(); 

			for(UINT i = 0; i < nItems; i++)
			{
				if(LVIS_SELECTED == m_pListCtrl->GetItemState(i, LVIS_SELECTED))
				{
					aProgerssCtrl.StepIt();

					CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(i);
					::_splitpath(pItemData->m_sourcePathStr, drive, dir, fname, ext);

					// if requested, make a copy of the source file in the source file's directory
					if(retVal == IDYES)
					{
						// find a different name for copy of the source file
						for(UINT nTry = 1; nTry < MAX_TRIES_TO_FIND_ARCHIVE_FILE_NAME; nTry++)
						{
							
							::sprintf(sourceCopyPathStr.GetBuffer(_MAX_PATH),
									  _T("%s%s%s[copy%d]%s"), drive, dir, fname, nTry, ext);

							sourceCopyPathStr.ReleaseBuffer();

							if(::CopyFile(pItemData->m_sourcePathStr, sourceCopyPathStr, TRUE))
							{
								nCopyMade++;
								break;
							}
						}

						if(nTry >= MAX_TRIES_TO_FIND_ARCHIVE_FILE_NAME) // failed to make copy
						{						
							if(bShowProblemMessage == IDOK)
							{
								CString aStr;
								aStr.Format(_T("Problem making a copy of source file %d\n\n%s%s\n\n%s"),
											aProgerssCtrl.GetPos(), fname, ext, GetLastErrorMessage());

								m_pListCtrl->EnsureVisible(i, FALSE);

								bShowProblemMessage = MessageBox(aStr, procStr, 
													  MB_ICONWARNING | MB_DEFBUTTON1 | MB_OKCANCEL);
							}

							sourceCopyPathStr = (_T(""));
						}
					}
					else
					{
						sourceCopyPathStr = (_T(""));
					}

					CString resutlStr;
					// restore source from archive
					if(::CopyFile(pItemData->m_archivePathStr, pItemData->m_sourcePathStr, FALSE))
					{
						nRestored++;
						resutlStr = _T("Restored"); // comment for RESULTS column
					}
					else
					{
						if(bShowProblemMessage == IDOK)
						{
							CString aStr;
							aStr.Format(_T("Problem restoring file %d\n\n%s%s\n\n%s"),
										i + 1, fname, ext, GetLastErrorMessage());

							m_pListCtrl->EnsureVisible(i, FALSE);

							bShowProblemMessage = MessageBox(aStr, procStr, 
								                  MB_ICONWARNING | MB_DEFBUTTON1 | MB_OKCANCEL);
						}
						// remove copy since couldn't restore
						if(!sourceCopyPathStr.IsEmpty()) 
						{
							::DeleteFile(sourceCopyPathStr);
							sourceCopyPathStr = _T("");
							nCopyMade--;
						}

						resutlStr = _T("Restore Failed"); // comment for RESULTS column
					}
					
					// add comment for RESULTS column
					if(!sourceCopyPathStr.IsEmpty())
					{
						CString aStr;
						aStr.Format(_T(", Source Copy = %s"),
							        sourceCopyPathStr.Mid(sourceCopyPathStr.ReverseFind(_T('\\')) + 1));

						resutlStr += aStr;
					}

					// write in RESULTS column
					if(!resutlStr.IsEmpty())
					{
						m_pListCtrl->SetItemText(i, RESULTS, resutlStr); 
					}
				}
			}
		} // end CProgressCtrl stack frame

		READY_MSG(); // show ready message in status bar

		CString aStr;
		aStr.Format(nRestored == 1 ? _T("%d File was restored") : _T("%d Files were restored"), nRestored);
		
		if(nCopyMade > 0)
		{
			CString bStr;
			bStr.Format(nCopyMade == 1 ? _T("\n%d Copy was made") : _T("\n%d Copies were made"), nCopyMade);
			aStr += bStr;
		}

		if(nRestored < nSel)
		{
			UINT nFailed = nSel - nRestored;
			CString bStr;
			bStr.Format(nFailed == 1 ? _T("\n%d Restore Failed") : _T("\n%d Restores Failed"), nFailed);
			aStr += bStr;
		}

		MessageBox(aStr, procStr, MB_ICONINFORMATION);
	}
}

void CDicksBackupView::OnUpdateEditRestore(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() > 0);	
}


void CDicksBackupView::OnHelpDoc() 
{
	CString strUrlSite = _T("http://dickinsonian.com/programs/DicksBackupDoc.pdf#page=1&view=fitH");

	if((HINSTANCE)32 >= ::ShellExecute(m_hWnd, NULL, strUrlSite, NULL, NULL, SW_SHOWNORMAL))
	{
		AfxMessageBox(GetLastErrorMessage() + "\n\n" + strUrlSite,  MB_ICONWARNING);  				
	}
}

void CDicksBackupView::OnEditOpenSource() 
{
	POSITION pos = m_pListCtrl->GetFirstSelectedItemPosition();

	if(pos)
	{
		int nItem = m_pListCtrl->GetNextSelectedItem(pos);
		CItemData * pItemData = (CItemData *)m_pListCtrl->GetItemData(nItem);
		CString pathStr = pItemData->m_sourcePathStr;
		CString file = pathStr.Mid(pathStr.ReverseFind('\\') + 1);
		CString dir = pathStr.Left(pathStr.ReverseFind('\\') + 1);

		if((HINSTANCE)32 >= ::ShellExecute(m_hWnd, NULL, file, NULL, dir, SW_SHOWNORMAL))
		{
			AfxMessageBox(GetLastErrorMessage() + "\n\n" + file,  MB_ICONWARNING);  				
		}
	}	
}

void CDicksBackupView::OnUpdateEditOpenSource(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pListCtrl->GetSelectedCount() == 1);	
}
