#include "stdafx.h"
#include "MyStuff.h"  

// start of CItemData class definition -------------------------------------------------------------------------

CItemData::CItemData(CString & sourcePathStr, CString & archivePathStr, CString & notesStr)
{
	m_sourcePathStr = sourcePathStr;
	m_archivePathStr = archivePathStr;
	m_notesStr = notesStr;

	m_lineFgColor = NOT_BACKED_UP_LIST_COLOR;
	m_lineBkColor = NOT_BACKED_UP_BKGND_LIST_COLOR;
	m_nSourceSize = 0;
	m_nArchiveSize = 0;
	m_modifiedTime = 0;
	m_archivedTime = 0;
	m_timeOutOfDate = 0;
	m_nlastError = 0;
	m_nExceptionIndicator = NONE;

	// get source file icon and type name
	SHFILEINFO sfi = {0};
	SHGetFileInfo(sourcePathStr, 0, &sfi, sizeof(sfi), SHGFI_ICON | SHGFI_TYPENAME);
	m_hIcon = sfi.hIcon;
	m_typeName = sfi.szTypeName;

	Refresh();
}

CItemData::~CItemData()
{
	if(m_hIcon)
	{
		DestroyIcon(m_hIcon);
	}
}

// returns TRUE if files properties have changed
void CItemData::Refresh()
{
	// reset archive values
	m_nArchiveSize  = 0;
	m_archivedTime  = 0;
	m_timeOutOfDate = 0;

	WIN32_FILE_ATTRIBUTE_DATA wfad;

	// check source file
	if(GetFileAttributesEx(m_sourcePathStr, GetFileExInfoStandard, &wfad))
	{
		m_nSourceSize = wfad.nFileSizeLow;
		m_modifiedTime = FileTimeToTimet(&wfad.ftLastWriteTime);
		m_lineFgColor = NOT_BACKED_UP_LIST_COLOR;
		m_lineBkColor = NOT_BACKED_UP_BKGND_LIST_COLOR;
		m_nExceptionIndicator = NONE;
		m_nlastError = 0;
	}
	else
	{
		m_nSourceSize = 0;
		m_modifiedTime = 0;
		m_lineFgColor = PROBLEM_LIST_COLOR;
		m_lineBkColor = PROBLEM_LIST_BKGND_COLOR;
		m_nExceptionIndicator = SOURCE;
		m_nlastError = GetLastError();
	}

	// source file OK so check archive file
	if(m_nExceptionIndicator == NONE)
	{
		if(GetFileAttributesEx(m_archivePathStr, GetFileExInfoStandard, &wfad))
		{
			m_nArchiveSize = wfad.nFileSizeLow;
			m_archivedTime = FileTimeToTimet(&wfad.ftLastWriteTime);

			// check times for out of date
			if(m_nArchiveSize == m_nSourceSize && m_archivedTime >= m_modifiedTime)
			{
				m_timeOutOfDate = 0;
				m_lineFgColor = BACKED_UP_LIST_COLOR;
				m_lineBkColor = BACKED_UP_BKGND_LIST_COLOR;
				m_nExceptionIndicator = NONE;
			}
			else
			{
				m_timeOutOfDate = m_modifiedTime - m_archivedTime; 
			}
		}
		else
		{
			m_lineFgColor = PROBLEM_LIST_COLOR;
			m_lineBkColor = PROBLEM_LIST_BKGND_COLOR;
			m_nExceptionIndicator = ARCHIVE;
			m_nlastError = GetLastError(); 
		}
	}
}

void CItemData::ChangeDrive(CString driveStr)
{
	char drive[_MAX_DRIVE], dir[_MAX_DIR], name[_MAX_FNAME], ext[_MAX_EXT];
	_splitpath(m_archivePathStr, drive, dir, name, ext);

	CString archivePathStr(driveStr);
	archivePathStr += dir;
	archivePathStr += name;
	archivePathStr += ext;
	TRACE("DRIVE [%s] to [%s]\n", m_archivePathStr, archivePathStr);
	m_archivePathStr = archivePathStr;
}


CString CItemData::GetTimeOutOfDate()
{
	return (m_modifiedTime > 0 && m_nArchiveSize > 0) ? GetDiffTime(m_timeOutOfDate) : CString(_T(""));
}

CString CItemData::GetDiffTime(time_t diffTime)
{
	CString aStr(_T(""));

	if(diffTime < 60)				{aStr.Format(_T("%d sec"), diffTime);}
	else if(diffTime < 3600)		{aStr.Format(_T("%d min"), diffTime / 60);}
	else if(diffTime < 86400)		{aStr.Format(_T("%d hour"), diffTime / 3600);}
	else if(diffTime < 604800)		{aStr.Format(_T("%d day"), diffTime / 86400);}
	else if(diffTime < 2592000)		{aStr.Format(_T("%d week"), diffTime / 604800);}
	else if(diffTime < 31536000)	{aStr.Format(_T("%d month"), diffTime / 2592000);} 
	else							{aStr.Format(_T("%d year"), diffTime / 31536000);} 

	return aStr;
}

CString CItemData::GetDrive()
{
	CString driveStr;

	if(m_sourcePathStr.Find(_T("\\\\") == 0)) // a network path
	{
		driveStr = m_sourcePathStr.Left(m_sourcePathStr.Find(_T("\\"), 2) + 1);
	}
	else // lettered drive
	{
		_splitpath(m_sourcePathStr, driveStr.GetBuffer(_MAX_DRIVE), NULL, NULL, NULL);
		driveStr.ReleaseBuffer();
	}

	return driveStr;
}

CString CItemData::GetFolder()
{
	CString folderStr;
	_splitpath(m_sourcePathStr, NULL, folderStr.GetBuffer(_MAX_DIR), NULL, NULL);
	folderStr.ReleaseBuffer();

	// strip trailing backslash
	int n = folderStr.ReverseFind(_T('\\'));

	if(n >= 0)
	{
		folderStr = folderStr.Left(n);	
	}

	// strip leading directories
	n = folderStr.ReverseFind(_T('\\'));

	if(n >= 0)
	{
		folderStr = folderStr.Mid(n + 1);	
	}

	return folderStr;
}

CString CItemData::GetName()
{
	CString fnameStr;
	_splitpath(m_sourcePathStr, NULL, NULL, fnameStr.GetBuffer(_MAX_FNAME), NULL);
	fnameStr.ReleaseBuffer();

	return fnameStr;
}

CString CItemData::GetType()
{
	CString extStr;
	_splitpath(m_sourcePathStr, NULL, NULL, NULL, extStr.GetBuffer(_MAX_FNAME));
	extStr.ReleaseBuffer();

	return extStr.Mid(extStr.Find(_T('.')) + 1); // drop the period at start of extension
}

CString CItemData::FormatDate(UINT dateFmtIndex)
{
	CString timeStr(_T(""));

	if(m_modifiedTime > 0)
	{
		struct tm * ptm = localtime(&m_modifiedTime);

		if(dateFmtIndex < AGE)
		{
			::strftime(timeStr.GetBuffer(sizeof(_T("01/01/2000 01:01:01 AM"))), 
					   sizeof(_T("01/01/2000 01:01:01 AM")), dateFmtStr[dateFmtIndex], ptm);
			
			timeStr.ReleaseBuffer();
		}
		else if(dateFmtIndex == AGE)
		{
			timeStr = GetDiffTime(CTime::GetCurrentTime().GetTime() - m_modifiedTime);
		}
	}

	return timeStr;
}

CString CItemData::FormatSize()
{
	CString aStr(_T(""));

	if(m_modifiedTime > 0)
	{
		ULONG nBytes = m_nSourceSize;
		UINT index = 0;

		if(nBytes >= 1000) // K
		{
			index++;
			nBytes /= 1000;
		}

		if(nBytes >= 1000) // M
		{
			index++;
			nBytes /= 1000;
		}

		if(nBytes >= 1000) // B
		{
			index++;
			nBytes /= 1000;
		}

		aStr.Format("%d %s", nBytes, byteStr[index]);
	}
	
	return aStr;
}

// end of CItemData class definition -------------------------------------------------------------------------

BOOL GetStrOnClipBoard(CString & aStr)
{
	if(::OpenClipboard(NULL))
	{
		HANDLE hData = ::GetClipboardData(CF_TEXT);
		
		if(hData)
		{
			ULONG nSize = GlobalSize(hData);

			if(nSize)
			{
				LPTSTR pBuf = aStr.GetBufferSetLength(nSize);
				LPSTR pData = (LPSTR)::GlobalLock(hData);

				::lstrcpy(pBuf, pData);

				aStr.ReleaseBuffer(-1);
				::GlobalUnlock(hData);
				::CloseClipboard();

				return TRUE;
			}
		}
	}

	return FALSE;
}

BOOL PutStrOnClipBoard(CString & aStr)	// overload on following proceedure using char *
{
	char * cBuf = aStr.GetBuffer(aStr.GetLength() + 1);

	BOOL bRet = PutStrOnClipBoard(cBuf);

	aStr.ReleaseBuffer(-1);

	return bRet;
}

BOOL PutStrOnClipBoard(char * cBuf)
{
	if(::OpenClipboard(NULL))	// copy to clipboard as aa text string
	{
		UINT len = ::strlen(cBuf) + 1;

		::EmptyClipboard();
		HANDLE hData = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, len);
		LPSTR pData = (LPSTR)::GlobalLock(hData);

		::CopyMemory(pData, cBuf, len);
		::GlobalUnlock(hData);
		::SetClipboardData(CF_TEXT, hData);
		::CloseClipboard();
		return TRUE;
	}

	return FALSE;
}

CString AddCRtoLF(CString & aStr)	// add CR to LF
{
	CString bStr(_T(""));

	int len = aStr.GetLength();

	for(int i = 0; i < len; i++)
	{
		TCHAR c = aStr[i];

		if(c != _T('\r'))
		{
			if(c == _T('\n'))
			{
				bStr += _T("\r\n");
			}
			else
			{
				bStr += c;
			}
		}
	}

	return bStr;
}

BOOL EkCreateProgressBar(	CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage,
							int nPaneIndex,	// if -1 use whole status bar width
							int cxMargin,
							int cxMaxWidth,
							UINT nIDControl)
{
	ASSERT_VALID( pProgressCtrl );
	ASSERT_VALID( pStatusBar );

	// 1 - Calculate destination rectangle for progress control
	int pIndx = nPaneIndex < 0 ? 0 : nPaneIndex;

    CRect rc;
    pStatusBar->GetItemRect(pIndx, &rc );

	if(nPaneIndex < 0)
	{
		CRect statusBarRect;
		pStatusBar->GetWindowRect(&statusBarRect);
		rc.right = rc.left + statusBarRect.Width();
	}

	// 2 - Define progress bar horizontal offset
	if( szMessage != NULL )
	{
		// Compute message text extent
		CClientDC dc( pStatusBar );
		CFont* pFont = pStatusBar->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CSize sizeText = dc.GetTextExtent( szMessage );
		dc.SelectObject( pOldFont );

		rc.left += sizeText.cx + cxMargin;
	}

	// 3 - Compute progress bar width
	if( cxMaxWidth != -1 )
	{
		rc.right = rc.left + min( cxMaxWidth, rc.Width() );
	}

	// 4 - Display message text
	pStatusBar->SetPaneText(pIndx, szMessage );
	pStatusBar->RedrawWindow();

	// 5 - Create progress control
    return pProgressCtrl->Create(PBS_SMOOTH | WS_CHILD | WS_VISIBLE,
									rc, pStatusBar, nIDControl );
}

void TimetToFileTime(time_t t, LPFILETIME pft)
{
    LONGLONG ll = Int32x32To64(t, 10000000) + 116444736000000000;
    pft->dwLowDateTime = (DWORD) ll;
    pft->dwHighDateTime = (DWORD)(ll >> 32);
}

time_t FileTimeToTimet(LPFILETIME pft)
{
	LONGLONG ll = (((LONGLONG)pft->dwHighDateTime) << 32) + ((LONGLONG)pft->dwLowDateTime);
	return (time_t)((ll - (LONGLONG)116444736000000000) / (LONGLONG)10000000);	
}

#include <Wininet.h>
CString GetLastErrorMessage()
{
	DWORD nErr = GetLastError();
	CString msgStr;

	if(nErr == ERROR_INTERNET_EXTENDED_ERROR)
	{
		DWORD dwError;
		char szBuffer[1024];
		DWORD dwBufferLength = sizeof(szBuffer);

		if(InternetGetLastResponseInfo(&dwError, szBuffer, &dwBufferLength))
		{
			 msgStr = szBuffer;
		}
	}
	else
	{
		LPVOID lpMsgBuf;

		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			nErr,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);

		msgStr = (LPCTSTR)lpMsgBuf;
		LocalFree(lpMsgBuf);
	}

	msgStr.TrimRight(); // remove CR-LF
	msgStr.TrimLeft();

	if(msgStr.IsEmpty())
	{
		msgStr.Format(_T("Last Error = %d"), nErr);
	}

	return msgStr;
}

int CALLBACK BrowseForFolderCallback(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData)
{
	char szPath[MAX_PATH];

	switch(uMsg)
	{
		case BFFM_INITIALIZED:
			SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
			break;

		case BFFM_SELCHANGED: 
			if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szPath)) 
			{
				SendMessage(hwnd, BFFM_SETSTATUSTEXT,0,(LPARAM)szPath);	

			}
			break;
	}

	return 0;
}

BOOL BrowseFolders(HWND hwnd, LPSTR lpszFolder, LPSTR lpszTitle)
{
	BROWSEINFO bi;
	char szPath[MAX_PATH + 1];
	LPITEMIDLIST pidl;
	BOOL bResult = FALSE;

	LPMALLOC pMalloc;

    if (SUCCEEDED(SHGetMalloc(&pMalloc))) 
	{
		bi.hwndOwner = hwnd;
		bi.pidlRoot = NULL;
		bi.pszDisplayName = NULL;
		bi.lpszTitle = lpszTitle;
		bi.ulFlags = BIF_RETURNONLYFSDIRS; //BIF_EDITBOX | BIF_STATUSTEXT
		bi.lpfn = BrowseForFolderCallback;
		bi.lParam = (LPARAM)lpszFolder;
		
		pidl = SHBrowseForFolder(&bi);
		if (pidl)
		{
			if (SHGetPathFromIDList(pidl,szPath)) 
			{
				bResult = TRUE;
				strcpy(lpszFolder, szPath);
			}

		   pMalloc->Free(pidl);
		   pMalloc->Release();
            			
		}
	}

	return bResult;	
}

