#ifndef MY_STUFF_H
#define MY_STUFF_H
#include "resource.h"

// font and toolbar stuff
UINT const _toolBarID[] = {IDR_TB_SMALL, IDR_TB_MEDIUM, IDR_MAINFRAME};
int const nPointSize[] = {10, 15, 20};
#define DFLT_FONT_PT_SIZE 15
#define DFLT_FONT_FACE_NAME _T("Arial")
enum {SMALL_FONT_POINT = 10, MEDIUM_FONT_POINT = 15, LARGE_FONT_POINT = 20};

// some colors
#define WHITE		RGB(0xff, 0xff, 0xff)
#define AQUA		RGB(0, 0xff, 0xff)
#define FUCHSIA		RGB(0xff, 0, 0xff)
#define YELLOW		RGB(0xff, 0xff, 0)
#define RED			RGB(0xff, 0, 0)
#define LIME		RGB(0, 0xff, 0)
#define BLUE		RGB(0, 0, 0xff)
#define SILVER		RGB(0xC0, 0xC0, 0xC0)
#define GRAY		RGB(0x80, 0x80, 0x80)
#define PRUPLE		RGB(0x80, 0, 0x80)
#define OLIVE		RGB(0x80, 0x80, 0)
#define	TEAL		RGB(0, 0x80, 0x80)
#define MAROON		RGB(0x80, 0, 0)
#define GREEN		RGB(0, 0x80, 0)
#define NAVY		RGB(0, 0, 0x80)
#define BLACK		RGB(0, 0, 0)

#define LT_GRAY		RGB(0Xf0, 0Xf0, 0Xf0) // used for alternating white-gray row backgroung stripes

// program colors
#define NO_DOC_BKGND_COLOR		       COLOR_MENU // sort of a gray

#define NOT_BACKED_UP_LIST_COLOR       BLACK
#define NOT_BACKED_UP_BKGND_LIST_COLOR WHITE

#define BACKED_UP_LIST_COLOR           GREEN
#define BACKED_UP_BKGND_LIST_COLOR     WHITE 

#define PROBLEM_LIST_COLOR             RED
#define PROBLEM_LIST_BKGND_COLOR       WHITE

// registry location in settings key
#define OSB _T("Opportunities in Small Business")
const char * const settings				= _T("Settings");
const char * const backupDirStr			= _T("backupDirStr");
const char * const addDirStr			= _T("addDirStr");
const char * const nFontPtSize			= _T("nFontPtSize");
const char * const bGrayBar				= _T("bGrayBar");
const char * const nAutoBackupTimeHr	= _T("nAutoBackupTimeHr");
const char * const bShowIcons			= _T("bShowIcons");
const char * const nDateFmt				= _T("nDateFmt");
const char * const bOpenMRU				= _T("bOpenMRU");

// file open / save stuff
#define MAX_TRIES_TO_FIND_NEW_FILE_NAME 25
#define BYTES_IN_MULTI_FILE_PATN_NAME_BUFFER 32768
const char * const versionString		   = _T("DicksBackup Version 1"); // don't change this it IDs a Backup File
const char * const noBackupTitle           = _T("Create or Open a Backup Manager");
const char * const myOpenAllFileTitle      = _T("Add Files to Backup Manager");
const char * const mySaveBackupFileTitle   = _T("Create Backup Manager");
const char * const myOpenBackupFileTitle   = _T("Open Backup Manager");
const char * const myImportBackupFileTitle = _T("Import Backup Manager Files");
const char * const myAllFileFilter		   = _T("All Files ( *.* )|*.*||");
const char * const myBackupFilter		   = _T("Backup Files ( *.bak )|*.bak|All Files ( *.* )|*.*||");
const char * const strIndicator            = _T(" *");
const char * const dfltExtension           = _T(".bak");
const UINT mySaveAsFlags = OFN_EXPLORER | OFN_ENABLESIZING | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;
const UINT myOpenFlags   = OFN_EXPLORER | OFN_ENABLESIZING | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT;

// browse for folders
BOOL BrowseFolders(HWND hwnd, LPSTR lpszFolder, LPSTR lpszTitle);

// error reporting
enum {NONE = 0, SOURCE = 1, ARCHIVE = 2};
const char * const sourceArchiveStr[] = {_T(""), _T("source "), _T("archive ")};

// persistent main window position
const char * const s_profileHeading	= _T("Main Window");
const char * const s_profileRect	= _T("Rect");
const char * const s_profileIcon	= _T("icon");
const char * const s_profileMax		= _T("max");
const char * const s_profileTool	= _T("tool");
const char * const s_profileStatus	= _T("status");

// list stuff
#define INITIAL_LIST_COUNT 100
enum {FILE_NAME, FILE_TYPE, FILE_SIZE, LAST_MODIFIED, SOURCE_PATH, ARCHIVE_PATH, RESULTS, OBSOLETE, NOTES, FOLDER, DRIVE, TYPE_NAME};
const char * const columnName[] = {_T("Name"), _T("Type"), _T("Size"), _T("Last Modified"), _T("Source Path"), 
								   _T("Archive Path"), _T("Results"), _T("Obsolete"), _T("Notes"), _T("Folder"), _T("Drive"), _T("Type Name")};
#define NCOLS NUM_ELEMENTS(columnName)

// holds data for each file
class CItemData
{
public:
	CString m_typeName;
	CItemData(CString & sourcePathStr, 
		      CString & archivePathStr = CString(_T("")), 
			  CString & notesStr = CString(_T("")));
	~CItemData();
	void Refresh();
	void ChangeDrive(CString driveStr);
	CString GetFolder();
	CString GetDrive();
	CString GetName();
	CString GetType();
	CString GetTimeOutOfDate();
	ULONG m_nSourceSize;
	ULONG m_nArchiveSize; 
	ULONG m_timeOutOfDate;
	time_t m_modifiedTime;
	time_t m_archivedTime;
	CString FormatDate(UINT dateFmt);
	CString FormatSize();
	CString m_sourcePathStr;
	CString m_notesStr;
	CString m_archivePathStr;
	UINT m_nlastError;
	UINT m_nExceptionIndicator;
	COLORREF m_lineFgColor;
	COLORREF m_lineBkColor;
	HICON m_hIcon;
protected:
	CString GetDiffTime(time_t diffTime);
};

// holds data for each column
enum _sortOrder {ASCENDING = 0, DECENDING = 1};

class CHdrExtraData
{
public:
	CHdrExtraData(UINT index) {m_nSortOrder = ASCENDING; m_index = index; 
							   m_bShowColumn = TRUE;}
	int m_nSortOrder; // ASCENDING or DECENDING
	UINT m_index; // the column index 
	BOOL m_bShowColumn;
};

template <class T> inline void Swap(T (& a), T (& b)) {T temp; temp = a; a = b; b = temp;}

// file size format
const char * const byteStr[] = {_T("B"), _T("KB"), _T("MB"), _T("GB")};
#define MAX_TRIES_TO_FIND_ARCHIVE_FILE_NAME 100

// email stuff
#define MY_EMAIL_URL "dick@dickinsonian.com"
#define SUBJECT "?subject=Concerning your program DicksBackup"
#define MAIL2 "mailto:"

// clip board stuff
BOOL PutStrOnClipBoard(char * cBuf);
BOOL PutStrOnClipBoard(CString & aStr);
BOOL GetStrOnClipBoard(CString & aStr);
CString AddCRtoLF(CString & aStr);

//Errors
CString GetLastErrorMessage();
 
// setup progress control macro
#define READY_MSG() ((CMainFrame *)AfxGetMainWnd())->PostMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE, 0);
#define NUM_PROGRESS_SEGMENTS 20	// number of segments in a fluu length progress bar
BOOL EkCreateProgressBar(CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage = NULL,
							int nPaneIndex = 0,
							int cxMargin = 10,
							int cxMaxWidth = -1,
							UINT nIDControl = 1);

#define PROGRESS(max, step, message) CWaitCursor wait;\
				 CProgressCtrl aProgerssCtrl;\
				 CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();\
				 ::EkCreateProgressBar(&aProgerssCtrl, &(pMainFrame->m_wndStatusBar), message, -1);\
				 aProgerssCtrl.SetRange32(0, max);\
				 aProgerssCtrl.SetStep(step);


// time conversions
void TimetToFileTime(time_t t, LPFILETIME pft);
time_t FileTimeToTimet(LPFILETIME pft);

// date format stuff
const char * const dateFmtStr[] = {_T("%d-%m-%Y %H:%M:%S"), _T("%m/%d/%Y %I:%M:%S %p"), _T("%Y-%m-%d %H:%M:%S"), _T("%H:%M:%S")};
enum {LITTLE_ENDIAN, MIDDLE_ENDIAN, BIG_ENDIAN, AGE};

// timer stuff
#define AUTO_BACKUP_TIMER_ID 1
#define DFLT_AUTO_BACKUP_TIMER_HOURS 24

// macros
#define NUM_ELEMENTS(x) (sizeof(x)/sizeof(x[0]))

// documentation
char const _documentationFile[] = _T("DicksBackupDoc.pdf");

#endif




